const request = require('request');
const AsksuiteProcessingNaturalUtils = require('../AsksuiteProcessingNaturalUtils');

class NLPClassifierAccessor {
  constructor(NLP_CLASSIFIER_LAYER_URL) {
    this.NLP_CLASSIFIER_LAYER_URL = NLP_CLASSIFIER_LAYER_URL;
    this.supportedLanguages = ['pt', 'es', 'en'];
    this.utils = new AsksuiteProcessingNaturalUtils();
  }

  resolveText(request) {
    return new Promise(resolve => {
      const executor = async () => {
        let response = null;
        let dialog = null;
        try {
          response = await this.requestData(this.NLP_CLASSIFIER_LAYER_URL, {
            phrase: request.text,
            language: request.language.substring(0, 2),
            intentType: 'cotacao',
          });
          dialog = this.utils.findDialogByIntent(request, response.body.intent);
          dialog.processingStep = {
            ...dialog.processingStep,
            text: request.text,
            language: request.language,
            resolver: 'NLP_CLASSIFIER',
            extraInfo: { ...response.body.trackingInfo },
          };
        } catch (e) {
          dialog = {
            processingStep: {
              fallbackReason: 'intent_not_found',
              intentFound: null,
              text: request.text,
              language: request.language,
              resolver: 'NLP_CLASSIFIER',
            },
          };
          console.log('[NLPClassifierAccessor error]', e);
        }
        resolve(dialog);
      };

      executor();
    });
  }

  requestData(url, json) {
    return new Promise((resolve, reject) => {
      request.post(url, { json }, (err, res) => {
        if (err) {
          reject(err);
        }
        resolve(res);
      });
    });
  }

  isSupportedLanguage(language) {
    return this.supportedLanguages.some(
      supportedLanguage => supportedLanguage === language.substring(0, 2),
    );
  }
}

module.exports = NLPClassifierAccessor;
